import os
import re

# 版本号  setup.py  ming.py 引用  每次发布 版本+1
version = '1.54'

# 配置目录存放在 用户根目录
root_config_dir = '{}/.jiuming-tools'.format(os.path.expanduser('~'))
if not os.path.exists(root_config_dir):
    os.makedirs(root_config_dir)

# 功能配置目录
config_default_file = '{}/.jiuming-tools/config'.format(os.path.expanduser('~'))
if not os.path.exists(config_default_file):
    os.makedirs(config_default_file)

# 脚本默认文件夹
script_default_file = '{}/.jiuming-tools/config/script'.format(os.path.expanduser('~'))
if not os.path.exists(script_default_file):
    os.makedirs(script_default_file)
# 正则
compile_ip = re.compile('^((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)$')
compile_host_mame = re.compile('^[a-zA-Z0-9][-a-zA-Z0-9]{0,62}(\.[a-zA-Z0-9][-a-zA-Z0-9]{0,62})+\.?$')



import os
import re
import subprocess

import click
import psutil


def echo_pc_info():
    """
    输出 设备产品信息
    :return:
    """
    os_info = os.uname()
    # 获取当前系统虚拟化方式
    virtual_type_str = str.split(subprocess.getoutput("lscpu | grep -E  '超管理器厂商|Hypervisor vendor'").strip(''))[1]

    memory_info_str = '''
操作系统:
  用户名:     {}
  主机名:     {}
  发行版本:   {}
  内核版本:   {}
  硬件架构:   {}
  虚拟化方式:  {}
cpu信息:
  cpu型号:    {}
  cpu物理核心: {}
  cpu逻辑核心: {}
  
内存信息:
{}

网卡信息:
{}

磁盘信息:
{}
        '''.format(
        os.getlogin(),
        os_info.nodename,
        subprocess.getoutput('cat /etc/issue').strip('\n'),
        os_info.release,
        os_info.machine,
        virtual_type_str if virtual_type_str is not None else "无",
        cpu_info().modelName,
        psutil.cpu_count(logical=False),
        psutil.cpu_count(),
        subprocess.getoutput('free -h'),
        subprocess.getoutput('ip addr'),
        subprocess.getoutput('df -h')
    )
    # 尝试执行 screenfetch
    try:
        res = subprocess.Popen("screenfetch", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        sout, serr = res.communicate()
        # res.returncode, sout, serr, res.pid
        if res.returncode == 0:
            memory_info_str += '\nscreenfetch:{}'.format(sout.decode('utf-8'))
    except OSError:
        pass
    click.echo(memory_info_str)


def cpu_info():
    """
    从 /proc/cpuinfo 中读取cpu 模型名称
    :return:
    """
    f_cpu_info = open('/proc/cpuinfo', 'r')
    c = f_cpu_info.readlines()
    for i in c:
        tmp_str_list = i.split(':')
        if re.match(".*model.*name.*", tmp_str_list[0]):
            return cpuInfo(tmp_str_list[1].strip('\n').strip('\t'))


class cpuInfo(object):
    def __init__(self, modelName):
        self.modelName = modelName


def byteToGb(byteNumber):
    # b   kb     mb      gb
    return str(format(byteNumber / 1024 / 1024 / 1024, '.3'))



import time


def count_time(info="计时"):
    """
    装饰函数   为函数装饰计算耗时
    :param info: 名称
    :return:
    """

    def _count_time(func):
        def _wrapper(*args, **kwargs):
            start_time = time.perf_counter()
            result = func(*args, **kwargs)
            end_time = time.perf_counter()
            print("{}耗时:{}μs,结果:{}".format(info, str(
                round((int(round((end_time - start_time) * 1000000))), 2)), str(result)))
            return result

        return _wrapper

    return _count_time



import socket


def get_host_ip():
    """
    获取 host ip
    :return: host ip
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(('8.8.8.8', 80))
    ip = s.getsockname()[0]
    s.close()
    return ip



import subprocess

import click
import speedtest


def testDisk(data_gb_size=2):
    """
     测试磁盘性能
    """
    # 块数量 =  data_size / 4k
    count = (data_gb_size * 1024 * 1024) / 4
    click.echo("测试写入速度中。。。。。。")
    click.echo(subprocess.getoutput('time dd if=/dev/zero of=./disk.test bs=4k count=%d' % count))
    click.echo("测试读取速度中。。。。。。")
    click.echo(subprocess.getoutput('time dd if=./disk.test of=/dev/null bs=4k'))
    click.echo(subprocess.getoutput('rm -rf ./disk.test'))
    click.echo("清理测试文件!完毕")


def testNetwork(threads=None):
    """
    测试网络速度
     参考地址: https://github.com/sivel/speedtest-cli/wiki
     results:格式{'download': 232349170.81585404, 'upload': 259741471.54632077, 'ping': 6.161, 'server': {'url': 'http://speedtest1.online.sh.cn:8080/speedtest/upload.php', 'lat': '31.2000', 'lon': '121.5000', 'name': 'Shanghai', 'country': 'China', 'cc': 'CN', 'sponsor': 'China Telecom', 'id': '3633', 'host': 'speedtest1.online.sh.cn:8080', 'd': 19.64397374170924, 'latency': 6.161}, 'timestamp': '2020-09-09T07:38:45.106069Z', 'bytes_sent': 151519232, 'bytes_received': 290975976, 'share': None, 'client': {'ip': '116.228.238.186', 'lat': '31.0449', 'lon': '121.4012', 'isp': 'China Telecom Shanghai', 'isprating': '3.7', 'rating': '0', 'ispdlavg': '0', 'ispulavg': '0', 'loggedin': '0', 'country': 'CN'}}
    """
    click.echo("speedtest测试网速中。。。。。。")
    servers = []
    s = speedtest.Speedtest()
    s.get_servers(servers)
    s.get_best_server()
    s.download(threads=threads)
    s.upload(threads=threads)
    results = s.results
    downloadMbits = float(results.download / 1000.0 / 1000.0)
    uploadMbits = float(results.upload / 1000.0 / 1000.0)
    click.echo('服务器名称: %(sponsor)s (%(name)s)\n距离:%(d)0.2f km\n延迟:%(latency)s ms' % results.server)
    click.echo('下载带宽: %0.2f Mbit/s\n下载速度: %0.2f MB/s' % (downloadMbits, downloadMbits / 8))
    click.echo('上传带宽: %0.2f Mbit/s\n上传速度: %0.2f MB/s' % (uploadMbits, uploadMbits / 8))
    click.echo('测速结果图片地址: %s' % results.share())
